@@ -23,20 +23,21 @@
 //! execute (as we do in the first, SynchronizeTrustedHash, phase) it would have taken more time and
 //! we might miss more eras.
 
+mod config;
 mod event;
 mod metrics;
 mod peers;
 mod state;
 mod traits;
 
-use std::{collections::BTreeMap, convert::Infallible, fmt::Display, mem, str::FromStr};
+use std::{convert::Infallible, fmt::Display, mem};
 
 use datasize::DataSize;
 use prometheus::Registry;
 use tracing::{error, info, trace, warn};
 
 use self::event::{BlockByHashResult, DeploysResult};
-use casper_types::{EraId, ProtocolVersion, PublicKey, U512};
+use casper_types::{EraId, ProtocolVersion};
 
 use super::{
     fetcher::FetchResult,
@@ -47,10 +48,12 @@ use crate::{
     effect::{EffectBuilder, EffectExt, EffectOptionExt, Effects},
     fatal,
     types::{
-        ActivationPoint, Block, BlockByHeight, BlockHash, Chainspec, FinalizedBlock, TimeDiff,
+        ActivationPoint, Block, BlockByHeight, BlockHash, BlockHeader, Chainspec, FinalizedBlock,
+        TimeDiff,
     },
     NodeRng,
 };
+pub use config::Config;
 use event::BlockByHeightResult;
 pub use event::Event;
 pub use metrics::LinearChainSyncMetrics;
@@ -71,11 +74,10 @@ pub(crate) struct LinearChainSync<I> {
     stop_for_upgrade: bool,
     /// Key for storing the linear chain sync state.
     state_key: Vec<u8>,
-    /// Acceptable drift between the block creation and now.
-    /// If less than than this has passed we will consider syncing as finished.
-    acceptable_drift: TimeDiff,
     /// Shortest era that is allowed with the given protocol configuration.
     shortest_era: TimeDiff,
+    /// Minimum round length that is allowed with the given protocol configuration.
+    min_round_length: TimeDiff,
     /// Flag indicating whether we managed to sync at least one block.
     started_syncing: bool,
     /// The protocol version the node is currently running with.
@@ -90,19 +92,18 @@ impl<I: Clone + PartialEq + 'static> LinearChainSync<I> {
         effect_builder: EffectBuilder<REv>,
         chainspec: &Chainspec,
         storage: &Storage,
-        init_hash: Option<BlockHash>,
+        trusted_hash: Option<BlockHash>,
         highest_block: Option<Block>,
-        _genesis_validator_weights: BTreeMap<PublicKey, U512>,
+        after_upgrade: bool,
         next_upgrade_activation_point: Option<ActivationPoint>,
+        config: Config,
     ) -> Result<(Self, Effects<Event<I>>), Err>
     where
         REv: From<Event<I>> + Send,
         Err: From<prometheus::Error> + From<storage::Error>,
     {
-        // set timeout to 5 minutes after now.
-        let five_minutes = TimeDiff::from_str("5minutes").unwrap();
         let timeout_event = effect_builder
-            .set_timeout(five_minutes.into())
+            .set_timeout(config.get_sync_timeout().into())
             .event(|_| Event::InitializeTimeout);
         let protocol_version = chainspec.protocol_config.version;
         if let Some(state) = read_init_state(storage, chainspec)? {
@@ -115,19 +116,32 @@ impl<I: Clone + PartialEq + 'static> LinearChainSync<I> {
             )?;
             Ok((linear_chain_sync, timeout_event))
         } else {
-            let acceptable_drift = chainspec.highway_config.max_round_length();
             // Shortest era is the maximum of the two.
             let shortest_era: TimeDiff = std::cmp::max(
                 chainspec.highway_config.min_round_length()
                     * chainspec.core_config.minimum_era_height,
                 chainspec.core_config.era_duration,
             );
-            let state = match init_hash {
-                Some(init_hash) => State::sync_trusted_hash(
-                    init_hash,
-                    highest_block.map(|block| block.take_header()),
-                ),
-                None => State::Done(highest_block.map(Box::new)),
+
+            let state = match trusted_hash {
+                Some(hash) => {
+                    State::sync_trusted_hash(hash, highest_block.map(|block| block.take_header()))
+                }
+                None if after_upgrade => {
+                    // Right after upgrade, no linear chain to synchronize.
+                    State::Done(highest_block.map(Box::new))
+                }
+                None => {
+                    if let Some(highest_block) = highest_block {
+                        // No trusted hash, not immediately after upgrade.
+                        // We will synchronize starting from the highest block we have.
+                        // NOTE: This is unsafe and should only use the highest block if it can
+                        // still be trusted – i.e. it's within the unbonding period.
+                        State::sync_descendants(*highest_block.hash(), highest_block, None)
+                    } else {
+                        State::Done(None)
+                    }
+                }
             };
             let state_key = create_state_key(chainspec);
             let linear_chain_sync = LinearChainSync {
@@ -137,8 +151,8 @@ impl<I: Clone + PartialEq + 'static> LinearChainSync<I> {
                 next_upgrade_activation_point,
                 stop_for_upgrade: false,
                 state_key,
-                acceptable_drift,
                 shortest_era,
+                min_round_length: chainspec.highway_config.min_round_length(),
                 started_syncing: false,
                 protocol_version,
             };
@@ -156,7 +170,6 @@ impl<I: Clone + PartialEq + 'static> LinearChainSync<I> {
     ) -> Result<Self, prometheus::Error> {
         let state_key = create_state_key(chainspec);
         info!(?state, "reusing previous state");
-        let acceptable_drift = chainspec.highway_config.max_round_length();
         // Shortest era is the maximum of the two.
         let shortest_era: TimeDiff = std::cmp::max(
             chainspec.highway_config.min_round_length() * chainspec.core_config.minimum_era_height,
@@ -169,8 +182,8 @@ impl<I: Clone + PartialEq + 'static> LinearChainSync<I> {
             next_upgrade_activation_point,
             stop_for_upgrade: false,
             state_key,
-            acceptable_drift,
             shortest_era,
+            min_round_length: chainspec.highway_config.min_round_length(),
             started_syncing: false,
             protocol_version,
         })
@@ -263,7 +276,7 @@ impl<I: Clone + PartialEq + 'static> LinearChainSync<I> {
         let hash = block.hash();
         trace!(%hash, %height, "downloaded linear chain block.");
         if block.header().is_switch_block() {
-            self.state.new_switch_block(&block);
+            self.state.set_last_switch_block_height(block.height());
         }
         if block.header().is_switch_block() && self.should_upgrade(block.header().era_id()) {
             info!(
@@ -313,7 +326,7 @@ impl<I: Clone + PartialEq + 'static> LinearChainSync<I> {
                 highest_block_seen,
                 trusted_hash,
                 ref latest_block,
-                maybe_switch_block,
+                last_switch_block_height,
                 ..
             } => {
                 assert_eq!(highest_block_seen, block_height);
@@ -336,12 +349,12 @@ impl<I: Clone + PartialEq + 'static> LinearChainSync<I> {
                 info!(%block_height, "Finished synchronizing linear chain up until trusted hash.");
                 let peer = self.peers.random_unsafe();
                 // Kick off syncing trusted hash descendants.
-                self.state = State::sync_descendants(trusted_hash, block, maybe_switch_block);
+                self.state = State::sync_descendants(trusted_hash, block, last_switch_block_height);
                 fetch_block_at_height(effect_builder, peer, block_height + 1)
             }
             State::SyncingDescendants {
                 ref latest_block,
-                ref maybe_switch_block,
+                last_switch_block_height,
                 ..
             } => {
                 if latest_block.as_ref() != &block {
@@ -351,22 +364,12 @@ impl<I: Clone + PartialEq + 'static> LinearChainSync<I> {
                     );
                     return fatal!(effect_builder, "unexpected block execution result").ignore();
                 }
-                if self.is_recent_block(&block) {
+                if self.is_currently_active_era(latest_block, last_switch_block_height) {
                     info!(
                         hash=?block.hash(),
                         height=?block.header().height(),
                         era=block.header().era_id().value(),
-                        "downloaded recent block. finished synchronization"
-                    );
-                    self.mark_done(Some(*latest_block.clone()));
-                    return Effects::new();
-                }
-                if self.is_currently_active_era(maybe_switch_block) {
-                    info!(
-                        hash=?block.hash(),
-                        height=?block.header().height(),
-                        era=block.header().era_id().value(),
-                        "downloaded switch block of a new era. finished synchronization"
+                        "downloaded a block in the current era. finished synchronization"
                     );
                     self.mark_done(Some(*latest_block.clone()));
                     return Effects::new();
@@ -377,18 +380,25 @@ impl<I: Clone + PartialEq + 'static> LinearChainSync<I> {
         }
     }
 
-    // Returns whether `block` can be considered the tip of the chain.
-    fn is_recent_block(&self, block: &Block) -> bool {
-        // Check if block was created "recently".
-        block.header().timestamp().elapsed() <= self.acceptable_drift
-    }
-
-    // Returns whether we've just downloaded a switch block of a currently active era.
-    fn is_currently_active_era(&self, maybe_switch_block: &Option<Box<Block>>) -> bool {
-        match maybe_switch_block {
-            Some(switch_block) => switch_block.header().timestamp().elapsed() < self.shortest_era,
-            None => false,
-        }
+    // Returns whether we've just downloaded a block in a currently active era.
+    fn is_currently_active_era(
+        &self,
+        block: &Block,
+        last_switch_block_height: Option<u64>,
+    ) -> bool {
+        let last_switch_block_height = last_switch_block_height.unwrap_or(0);
+        // This is the number of blocks that have we already know of from the era the current block
+        // is in.
+        let past_blocks_in_this_era = block.height().saturating_sub(last_switch_block_height);
+        // `self.shortest_era - self.min_round_length * past_blocks_in_this_era` is the estimated
+        // time left to the end of the era the current block is in; if less time than that has
+        // passed since the current block, this is most likely still the current era and we can
+        // return `true`.
+        // We add `min_round_length * past_blocks_in_this_era` to the left side instead of
+        // subtracting it from the right side to avoid underflows (TimeDiffs can't represent values
+        // less than 0).
+        block.header().timestamp().elapsed() + self.min_round_length * past_blocks_in_this_era
+            < self.shortest_era
     }
 
     /// Returns effects for fetching next block's deploys.
@@ -503,6 +513,17 @@ impl<I: Clone + PartialEq + 'static> LinearChainSync<I> {
         }
     }
 
+    pub fn into_maybe_latest_block_header(self) -> Option<BlockHeader> {
+        match self.state {
+            State::SyncingTrustedHash { latest_block, .. } => latest_block.map(Block::take_header),
+            State::SyncingDescendants { latest_block, .. } => Some(latest_block.take_header()),
+            State::Done(maybe_latest_block) => {
+                maybe_latest_block.map(|latest_block| latest_block.take_header())
+            }
+            State::None => None,
+        }
+    }
+
     fn should_upgrade(&self, era_id: EraId) -> bool {
         match self.next_upgrade_activation_point {
             None => false,
